#ifndef __RADIXSORT_H
#define __RADIXSORT_H

#include <cstring>
#include <stdint.h>
#include <algorithm>

template <typename Iterator, typename Compare>
static inline void radixsort8msb_copy2(Iterator begin, Iterator end, size_t depth, Compare& cmp) {
    typedef typename Iterator::value_type value_type;
    if (end - begin < 128) {
        std::sort(begin, end, cmp);
        return;
    }

    const size_t K = 256;
    size_t bucketsize[K];
    memset(bucketsize, 0, K * sizeof(bucketsize[0]));

    // fill oracle and count character occurances
    uint8_t* oracle = (uint8_t*)malloc((end - begin) * sizeof(uint8_t));
    size_t ic = 0;
    for (Iterator i = begin; i != end; ++i, ++ic) {
        uint8_t v = ((uint8_t*)&(*i))[depth];
        ++bucketsize[ oracle[ic] = v ];
    }

    // prefix sum
    ssize_t bucketindex[K];
    bucketindex[0] = 0;
    for (size_t i = 1; i < K; ++i)
        bucketindex[i] = bucketindex[i-1] + bucketsize[i-1];

    // out-of-place permutation
    value_type* sorted = new value_type[end - begin];
    ic = 0;
    for (Iterator i = begin; i != end; ++i, ++ic)
        sorted[ bucketindex[ oracle[ic] ]++ ] = *i;

    free(oracle);
    std::copy(sorted, sorted + (end - begin), begin);
    delete [] sorted;

    if (depth == 0) return;

    // recursion into bucket
    Iterator i = begin;
    for (size_t j = 0; j < K; i += bucketsize[j++]) {
        if (bucketsize[j] <= 1) continue;
        radixsort8msb_copy2(i, i + bucketsize[j], depth-1, cmp);
    }
}

#endif // __RADIXSORT_H
