#ifndef __UTILS_H
#define __UTILS_H

#include <cstdio>
#include <cstdlib>
#include <unistd.h>
#include <sys/time.h>

#include <string>
#include <algorithm>

namespace utils {

#define STRX(x) #x
#define STR(x) STRX(x)

/******************************* SYSTEM CALLS *********************************/

void execute(std::string cmd) {
  int system_ret = system(cmd.c_str());
  if (system_ret) {
    fprintf(stderr, "Error: executing command [%s] returned %d.\n",
        cmd.c_str(), system_ret);
    std::exit(EXIT_FAILURE);
  }
}

/****************************** MEASURING TIME ********************************/

long double wclock() {
  timeval tim;
  gettimeofday(&tim, NULL);

  return tim.tv_sec + (tim.tv_usec / 1000000.0L);
}

/**************************** FILE MANIPULATION *******************************/

FILE *open_file(std::string fname, std::string mode) {
  FILE *f = std::fopen(fname.c_str(), mode.c_str());
  if (!f) {
    std::perror(fname.c_str());
    std::exit(EXIT_FAILURE);
  }

  return f;
}

long file_size(std::string fname) {
  FILE *f = open_file(fname, "rt");
  std::fseek(f, 0, SEEK_END);
  long size = std::ftell(f);
  std::fclose(f);

  return size;
}

bool file_exists(std::string fname) {
  FILE *f = std::fopen(fname.c_str(), "r");
  bool ret = (f != NULL);

  if (f != NULL)
    std::fclose(f);

  return ret;
}

template<typename value_type>
void add_objects_to_file(value_type *tab, long length, FILE *f) {
  size_t fwrite_ret = fwrite(tab, sizeof(value_type), length, f);
  if ((long)fwrite_ret != length) {
    fprintf(stderr, "Error: fwrite in line %s of %s returned %lu\n",
        STR(__LINE__), STR(__FILE__), fwrite_ret);
    std::exit(EXIT_FAILURE);
  }
}

template<typename T>
T read_from_pos(FILE *f, long pos) {
  std::fseek(f, pos, SEEK_SET);
  T res;
  
  size_t fread_ret = fread(&res, sizeof(T), 1, f);
  if (fread_ret != 1) {
    fprintf(stderr, "Error: fread in line %s of %s returned %lu\n", 
        STR(__LINE__), STR(__FILE__), fread_ret);
    std::exit(EXIT_FAILURE);
  }
  
  return res;
}

/********************************* MISC ***************************************/

void find_stxxl_config() {
  if (file_exists("./.stxxl")) {
    fprintf(stderr, "STXXL config file detected.\n");
    return;
  } else if (file_exists(std::string(getenv("HOME")) + "/.stxxl")) {
    fprintf(stderr, "Cannot find STXXL config file. Using $HOME/.stxxl\n");
    execute("cp " + std::string(getenv("HOME")) + "/.stxxl ./");
    return;
  } else {  
    fprintf(stderr, "Error: failed to find/copy STXXL config file!\n");
    std::exit(EXIT_FAILURE);
  }
}

} // namespace utils

#endif // __UTILS_H
